@@ -1,5 +1,5 @@
 /*
-**  Copyright (c) 2012, 2013, The Trusted Domain Project.  All rights reserved.
+**  Copyright (c) 2012-2014, The Trusted Domain Project.  All rights reserved.
 */
 
 #ifndef _OPENDMARC_CONFIG_H_
@@ -28,7 +28,9 @@ struct configdef dmarcf_config[] =
 	{ "DNSTimeout",			CONFIG_TYPE_INTEGER,	FALSE },
 	{ "EnableCoredumps",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "ForensicReports",		CONFIG_TYPE_BOOLEAN,	FALSE },
+	{ "ForensicReportsBcc",		CONFIG_TYPE_STRING,	FALSE },
 	{ "ForensicReportsOnNone",	CONFIG_TYPE_BOOLEAN,	FALSE },
+	{ "ForensicReportsSentBy",	CONFIG_TYPE_STRING,	FALSE },
 	{ "HistoryFile",		CONFIG_TYPE_STRING,	FALSE },
 	{ "IgnoreHosts",		CONFIG_TYPE_STRING,	FALSE },
 	{ "IgnoreMailFrom",		CONFIG_TYPE_STRING,	FALSE },
@@ -36,10 +38,11 @@ struct configdef dmarcf_config[] =
 	{ "PidFile",			CONFIG_TYPE_STRING,	FALSE },
 	{ "PublicSuffixList",		CONFIG_TYPE_STRING,	FALSE },
 	{ "RecordAllMessages",		CONFIG_TYPE_BOOLEAN,	FALSE },
+	{ "RequiredHeaders",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "RejectFailures",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "ReportCommand",		CONFIG_TYPE_STRING,	FALSE },
 	{ "Socket",			CONFIG_TYPE_STRING,	FALSE },
-	{ "SoftwareHeader",		CONFIG_TYPE_STRING,	FALSE },
+	{ "SoftwareHeader",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "Syslog",			CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "SyslogFacility",		CONFIG_TYPE_STRING,	FALSE },
 	{ "TemporaryDirectory",		CONFIG_TYPE_STRING,	FALSE },
